/* Copyright (C) 2011 SpringSource
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.grails.datastore.mapping.jpa.config;

import java.lang.reflect.Field;
import java.util.List;

import javax.persistence.Column;
import javax.persistence.Table;

import groovy.lang.Closure;
import org.grails.datastore.mapping.config.groovy.MappingConfigurationBuilder;
import org.grails.datastore.mapping.model.MappingFactory;
import org.grails.datastore.mapping.model.PersistentEntity;
import org.grails.datastore.mapping.model.PersistentProperty;
import org.grails.datastore.mapping.model.config.GormProperties;
import org.grails.datastore.mapping.reflect.ClassPropertyFetcher;
import org.springframework.beans.BeanUtils;

/**
 * MappingFactory for JPA that maps entities to {@link Table} instances and properties
 * to {@link Column} instances.
 *
 * @author Graeme Rocher
 * @since 1.0
 */
@SuppressWarnings({"rawtypes", "unchecked"})
public class JpaMappingFactory extends MappingFactory<org.grails.datastore.mapping.jpa.config.Table, org.grails.datastore.mapping.jpa.config.Column> {

    @Override
    public org.grails.datastore.mapping.jpa.config.Table createMappedForm(PersistentEntity entity) {
        org.grails.datastore.mapping.jpa.config.Table mappedForm = new org.grails.datastore.mapping.jpa.config.Table((Table) entity.getJavaClass().getAnnotation(Table.class));
        ClassPropertyFetcher cpf = ClassPropertyFetcher.forClass(entity.getJavaClass());
        MappingConfigurationBuilder builder = new MappingConfigurationBuilder(mappedForm, org.grails.datastore.mapping.jpa.config.Table.class);

        List<Closure> values = cpf.getStaticPropertyValuesFromInheritanceHierarchy(GormProperties.MAPPING, Closure.class);
        for (int i = values.size(); i > 0; i--) {
            Closure value = values.get(i - 1);
            builder.evaluate(value);
        }
        values = cpf.getStaticPropertyValuesFromInheritanceHierarchy(GormProperties.CONSTRAINTS, Closure.class);
        for (int i = values.size(); i > 0; i--) {
            Closure value = values.get(i - 1);
            builder.evaluate(value);
        }

        return mappedForm;
    }

    @Override
    public org.grails.datastore.mapping.jpa.config.Column createMappedForm(PersistentProperty mpp) {
        Field field;
        try {
            field = mpp.getOwner().getJavaClass().getDeclaredField(mpp.getName());
            return new org.grails.datastore.mapping.jpa.config.Column(field.getAnnotation(Column.class));
        } catch (SecurityException e) {
            return null;
        } catch (NoSuchFieldException e) {
            return null;
        }
    }
}
